@@ -17,7 +17,8 @@
 #' \item   plr                    Positive Likelihood Ratio   
 #' \item   nlr                    Negative Likelihood Ratio  
 #' \item   typeI.error            Type I error (omission)
-#' \item   typeII.error           Type II error (comission)
+#' \item   typeII.error           Type II error (commission)
+#' \item   gini                   Gini entropy index
 #' \item   f.score                F-score
 #' \item   gain                   Information gain (aka precision)
 #' \item   mcc                    Matthew's correlation 
@@ -37,6 +38,7 @@
 #'   \item   F-Score = 2 * (precision * recall) / (precision + recall) 
 #'   \item   Hanssen-Kuiper skill score (aka true score statistic) = [(tp * tn) - (fp * fn)] / [(tp + fn) + (fp + tn)], The true skill score has an expected -1 to +1, with 0 representing no discrimination.   
 #'  } 
+#' @note Using the table function matrix positions for a 2x2 confusion matrix are TP(1), FN(3), FP(2), TN(4)
 #'
 #' @author Jeffrey S. Evans    <jeffrey_evans<at>tnc.org>
 #'
@@ -63,9 +65,11 @@ accuracy <- function (x, y) {
   } else {
     t.xy <- table(x, y)
   }
-    if(inherits(t.xy, "matrix")) { 
-      rownames(t.xy) <- c("0","1")
-	  colnames(t.xy) <- c("0","1")
+    if(inherits(t.xy, "matrix")) {
+      if(is.null(rownames(t.xy))) {	
+        rownames(t.xy) <- c("0","1")
+	    colnames(t.xy) <- c("0","1")
+	  }
     }
 	tc <- match(colnames(t.xy), rownames(t.xy))
 	mtc <- matrix(ncol = ncol(t.xy), nrow = length(tc[tc == "NA"]), 0)
@@ -85,56 +89,69 @@ accuracy <- function (x, y) {
     t4 <- t(t3[mc, ])
       agree <- diag(t4)
         prod1 <- apply(t4, 1, sum)
-          prod2 <- agree/prod1
+          prod2 <- agree / prod1
              user1 <- apply(t4, 2, sum)
-            user2 <- agree/user1
+            user2 <- agree / user1
           N <- sum(t4)
         k1 <- sum(agree)
       k2 <- sum(prod1 * user1)
-    khat <- abs(((N * k1) - k2)/(N^2 - k2))
-	
-    if( dim(t.xy)[1] == 2 ) {	
-      n = sum(t.xy)
-      # prevalence <- t.xy[1] / n 
-      precision <- t.xy[1] / (t.xy[1] + t.xy[3])  
-      tpr <- t.xy[1] / ( t.xy[1] +  t.xy[2] )    
-      tnr <- t.xy[4] / ( t.xy[4] +  t.xy[3] )    
-      type1.error <- 1 - tnr                     
+    khat <- abs(((N * k1) - k2) / (N^2 - k2))	
+    if( dim(t.xy)[1] == 2 ) {
+      # TP(1)  FN(3)  
+      # FP(2)  TN(4)  	
+      n = sum(t.xy)  # N	
+      TP <- t.xy[1]  # True Positives, Power 
+	  FP <- t.xy[2]  # False Positives, Type-I error 
+	  FN <- t.xy[3]  # False Negatives, Type-II error 
+	  TN <- t.xy[4]  # True Negatives
+      # prevalence <- TP / n
+      precision <- TP / (TP + FP)  	  
+	  tpr <- TP / (TP + FN)  # true positive rate (aka, sensitivity, recall)
+	  tnr <- TN / (TN + FP)  # true negative rate (aka, specificity, selectivity)	 
+	  fpr <- FP / (FP + TN) 
+	  fnr <- FN / (FN + TP)  # Beta       
+	  type1.error <- 1 - tnr                     
       type2.error <- 1 - tpr                     
       plr <- tpr / (1 - tnr)                     
-      nlr <- (1 - tpr) / tnr                     
-      f.score <- 2 * (precision * tpr) / (precision + tpr)
+      nlr <- (1 - tpr) / tnr 
+	  auc <- (tpr - fpr + 1) / 2
+	  gini <- 2 * auc - 1  
+	  f.score <- 2 * (precision * tpr) / (precision + tpr)
 	  true.skill <- ( (t.xy[1] * t.xy[4]) - (t.xy[3] * t.xy[2]) ) / 
-	                  ( (t.xy[1] + t.xy[2]) * (t.xy[3] + t.xy[4]) )
-	  gain <- precision / ( (t.xy[1] + t.xy[4]) / n ) 
-	  mcc <- ( t.xy[1] * t.xy[4] - t.xy[3] * t.xy[2] ) /
-              sqrt( (t.xy[1] + t.xy[3]) * 
-			    (t.xy[1] + t.xy[2]) * 
-			    (t.xy[4] + t.xy[3]) * 
-			    (t.xy[4] + t.xy[2]) )
-      auc <- (tpr - ( t.xy[3] / ( t.xy[2] +  t.xy[3] ) ) + 1) / 2
+	                ( (t.xy[1] + t.xy[2]) * (t.xy[3] + t.xy[4]) )
+	  gain <- precision / ( (t.xy[1] + t.xy[4]) / n )
+	  mcc <- (TP * TN - FP * FN) / sqrt( (TP + FP) * (TP + FN) * 
+			 (TN + FP) * (TN + FN) )			 
+		confusion <- matrix(c(paste0("True positive(", TP, ")"),
+		         paste0("False positive(", FP, ")"),
+		         paste0("False negative(", FN, ")"),
+		         paste0("True negative(", TN, ")")),
+				 nrow=2, byrow=TRUE)
+          rownames(confusion) <- rownames(t.xy)
+		  colnames(confusion) <- colnames(t.xy)
 	  acc <- list( PCC = (sum(diag(t4))/sum(t4)) * 100,
-                 auc = auc, 	
-	             users.accuracy = round(user2 * 100, 1),  
-	             producers.accuracy = round(prod2 * 100, 1),
-                 kappa = round(khat, 4),
-				 true.skill = true.skill, 
-				 sensitivity = tpr,
-				 specificity = tnr,
-				 plr = plr,
-				 nlr = nlr,
-				 typeI.error = type1.error,
-				 typeII.error = type2.error,
-				 f.score = f.score,
-				 gain = gain,
-                 matthews = mcc, 				 
-                 confusion = t.xy)	
-        } else {
+                   auc = auc, 	
+	               users.accuracy = round(user2 * 100, 1),  
+	               producers.accuracy = round(prod2 * 100, 1),
+                   kappa = round(khat, 4),
+				   true.skill = true.skill, 
+				   sensitivity = tpr,
+				   specificity = tnr,
+				   plr = plr,
+				   nlr = nlr,
+				   typeI.error = type1.error,
+				   typeII.error = type2.error,
+				   gini = gini,
+				   f.score = f.score,
+				   gain = gain,
+                   matthews = mcc,
+				   confusion = confusion )	
+    } else {
 	  acc <- list( PCC = (sum(diag(t4))/sum(t4)) * 100, 
 	             users.accuracy = round(user2 * 100, 1),  
 	             producers.accuracy = round(prod2 * 100, 1),
                  kappa = round(khat, 4),
-                 confusion = t.xy)
+                 confusion = t.xy )
 	}			 
 	class(acc) <- c("accuracy", "list") 			   
           return( acc )
